@@ -17,7 +17,6 @@
 #
 
 import httplib
-import mock
 
 
 def _conn_str(conn):
@@ -25,8 +24,6 @@ def _conn_str(conn):
         proto = "https://"
     elif isinstance(conn, httplib.HTTPConnection):
         proto = "http://"
-    elif isinstance(conn, mock.Mock):
-        proto = "http://"
     else:
         raise TypeError('_conn_str() invalid connection type: %s' % type(conn))
 
