@@ -179,6 +179,7 @@ class TestBasicRouterOperations(unittest.TestCase):
     def testProcessRouter(self):
 
         agent = l3_agent.L3NATAgent(self.conf)
+        agent.client_create()
         router_id = _uuid()
         ri = l3_agent.RouterInfo(router_id, self.conf.root_helper,
                                  self.conf.use_namespaces)
@@ -231,6 +232,7 @@ class TestBasicRouterOperations(unittest.TestCase):
 
     def testSingleLoopRouterRemoval(self):
         agent = l3_agent.L3NATAgent(self.conf)
+        agent.client_create()
 
         self.client_inst.list_ports.return_value = {'ports': []}
 
@@ -273,12 +275,37 @@ class TestBasicRouterOperations(unittest.TestCase):
             def __init__(self, name):
                 self.name = name
 
-        self.mock_ip.get_namespaces.return_value = ['qrouter-foo']
+        self.mock_ip.get_namespaces.return_value = ['qrouter-foo',
+                                                    'qrouter-bar']
         self.mock_ip.get_devices.return_value = [FakeDev('qr-aaaa'),
                                                  FakeDev('qgw-aaaa')]
 
         agent = l3_agent.L3NATAgent(self.conf)
-        agent._destroy_all_router_namespaces()
+        agent._destroy_router_namespace = mock.MagicMock()
+        agent._destroy_router_namespaces()
+
+        self.assertEqual(agent._destroy_router_namespace.call_count, 2)
+
+    def testDestroyNamespaceWithRouterId(self):
+
+        class FakeDev(object):
+            def __init__(self, name):
+                self.name = name
+
+        self.conf.router_id = _uuid()
+
+        namespaces = ['qrouter-foo', 'qrouter-' + self.conf.router_id]
+
+        self.mock_ip.get_namespaces.return_value = namespaces
+        self.mock_ip.get_devices.return_value = [FakeDev('qr-aaaa'),
+                                                 FakeDev('qgw-aaaa')]
+
+        agent = l3_agent.L3NATAgent(self.conf)
+
+        agent._destroy_router_namespace = mock.MagicMock()
+        agent._destroy_router_namespaces(self.conf.router_id)
+
+        self.assertEqual(agent._destroy_router_namespace.call_count, 1)
 
     def testMain(self):
         agent_mock_p = mock.patch('quantum.agent.l3_agent.L3NATAgent')
